function id=jobsubmit(options)

% missing options
if ~isfield(options,'frontend')
  error('ERROR: Must specify frontend node')
end

if ~isfield(options,'remdir')
  error('ERROR: Must specify path to remote directory')
end

if ~isfield(options,'locdir')
  error('ERROR: Must specify path to local directory')
end

if ~isfield(options,'subscript')
  error('ERROR: Must specify job submission script')
end

if ~isfield(options,'execmd')
  error('ERROR: Must specify command to execute')
end

% Set defaults
if ~isfield(options,'system')
  options.system='PBS';
end

if ~isfield(options,'checktype')
  options.checktype=1;
end
if ~ismember(options.checktype,[1 2])
  error('ERROR: options.checktype must be 1 or 2')
end
if options.checktype==1  
  if ~isfield(options,'checkfile')
    options.checkfile='donefile';  
  end
elseif options.checktype==2
  if ~isfield(options,'checktoken')
    error('ERROR: Must specify checktoken option for job check')
  end
  if ~isfield(options,'checkfile')
	options.checkfile='logfile';
  end
  if ~isempty(strfind(options.execmd,'>')) % make sure we can redirect stdout to checkfile below
	error('ERROR: Found ">" in execmd!')
  end
end
if ~isfield(options,'checkcommandfile')
  options.checkcommandfile='checkjob';
end

if ~isfield(options,'scriptname')
  options.scriptname='jobscript';
end

if ~isfield(options,'sharedfilesystem')
  options.sharedfilesystem=0;
end
if ~ismember(options.sharedfilesystem,[0 1])
  error('ERROR: options.sharedfilesystem must be 0 or 1')
end

if ~isfield(options,'singlemachine')
  options.singlemachine=0;
end
if ~ismember(options.singlemachine,[0 1])
  error('ERROR: options.singlemachine must be 0 or 1')
end
if options.singlemachine
  if ~strcmp(options.locdir,options.remdir)
    error('ERROR: Local and remote paths differ even though options.singlemachine is set to 1')
  end
  if ~options.sharedfilesystem
    error('ERROR: options.singlemachine is set to 1, but options.sharedfilesystem is set to 0')
  end    
end  

if ~isfield(options,'sshport')
  sshport='';
  scpport='';
else
  sshport=['-p ' options.sshport];
  scpport=['-P ' options.sshport];
end  

currDir=pwd;

% create all local files in options.locdir
cd(options.locdir)

if ~options.sharedfilesystem  % make a directory to hold temporary files
  tmpDir=['TMPDIR' num2str(round(rand(1)*100000))];
  evalExternalCommand(['mkdir ' tmpDir]);
  cd(tmpDir)
end

% Generate script for checking job status
if options.checktype==1  % use existence of file to check status
  fid = fopen(options.checkcommandfile,'w');
  fprintf(fid,'%s\n','#!/bin/tcsh');
  fprintf(fid,'%s\n',['if (-e ' options.checkfile ') then']);
  fprintf(fid,'%s\n','  echo "1"');
  fprintf(fid,'%s\n','else');
  fprintf(fid,'%s\n','  echo "0"');
  fprintf(fid,'%s\n','endif');
  fclose(fid);
  evalExternalCommand(['chmod +x ' options.checkcommandfile]);
elseif options.checktype==2 % grep for token in checkfile
  fid = fopen(options.checkcommandfile,'w');
  fprintf(fid,'%s\n','#!/bin/tcsh');
  fprintf(fid,'%s\n',['if (-e ' options.checkfile ') then']);
  fprintf(fid,'%s\n',['  set out=`grep "' options.checktoken '" ' options.checkfile '`']);
  fprintf(fid,'%s\n','  set exitstatus=$?');
  fprintf(fid,'%s\n','  if ($exitstatus == "0") then');
  fprintf(fid,'%s\n','    echo "1"');
  fprintf(fid,'%s\n','  else');
  fprintf(fid,'%s\n','    echo "0"');
  fprintf(fid,'%s\n','  endif');
  fprintf(fid,'%s\n','else');
  fprintf(fid,'%s\n','  echo "0"');
  fprintf(fid,'%s\n','endif');  
  fclose(fid);
  evalExternalCommand(['chmod +x ' options.checkcommandfile]);
end

fid = fopen(options.scriptname,'w');
if iscell(options.subscript)
  for ie=1:length(options.subscript)
    fprintf(fid,'%s\n',options.subscript{ie});
  end
else
  fprintf(fid,'%s\n',options.subscript);
end

if isfield(options,'extracmds')
  if iscell(options.extracmds)
	for ie=1:length(options.extracmds)
	  fprintf(fid,'%s\n',options.extracmds{ie});
	end
  else
	fprintf(fid,'%s\n',options.extracmds);
  end
end

if options.checktype==2
  fprintf(fid,'%s\n',[options.execmd ' > ' options.checkfile]);
else
  fprintf(fid,'%s\n',options.execmd);
end

if options.checktype==1  
  fprintf(fid,'%s\n',['touch ' options.checkfile]);
end
fclose(fid);

%%

if ~options.sharedfilesystem % scp files over to frontend and delete temporary directory
  evalExternalCommand(['scp ' scpport ' -p ' options.scriptname ' ' options.frontend ':' options.remdir]);
  evalExternalCommand(['scp ' scpport ' -p ' options.checkcommandfile ' ' options.frontend ':' options.remdir]);  
  if options.checktype==1 % delete preexisting checkfile 
    evalExternalCommand(['ssh -n ' sshport ' ' options.frontend ' "cd ' options.remdir ';rm -f ' options.checkfile '"']);
  elseif options.checktype==2 % delete preexisting checkfile
    evalExternalCommand(['ssh -n ' sshport ' ' options.frontend ' "cd ' options.remdir ';rm -f ' options.checkfile '"']);
  end  
  cd(options.locdir)
  evalExternalCommand(['rm -rf ' tmpDir]);
else  
  if options.checktype==1 % delete preexisting checkfile
    evalExternalCommand(['rm -f ' options.checkfile]);
  elseif options.checktype==2 % delete preexisting checkfile
    evalExternalCommand(['rm -f ' options.checkfile]);
  end  
end

if ~options.singlemachine
  remJobStr=['ssh -n ' sshport ' ' options.frontend ' "cd ' options.remdir ';' options.subcmd ' ' options.scriptname '"'];
else
  cd(options.locdir) % to be safe
  remJobStr=[options.subcmd ' ' options.scriptname];
end
[id,err]=evalExternalCommand(remJobStr);

cd(currDir)
